/* 
 * glDino - 3D motor to render three-dimentional objects
 *
 * Mesa 3D graphics library 2.6 version
 * Sun Solaris 2.5 or higher
 * Linux 2.0.x or higher
 *
 * Authors: Navarro Mikael (navarro@essi.fr)
 *          David Farrell
 *
 */

/*
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of OpenGL Inc. not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Compile: gcc -ansi -O2 -o glDino glDino.c camera.c 
   -L/usr/X11R6/lib -lglut -lGLU -lGL -lXmu -lXext -lX11 -lm */

#ifdef WIN32
#include "windows.h"
#endif

#include <stdio.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "camera.h" /* by David Farell */

/*
 * Include the .h file with the display list here
 * Since each file has unique function names, you
 * can include as many of the display lists here
 * as you want. Be sure the corresponding .gl files
 * are in the current directory.
 *
 */
#include "hubble.h"

/* Globals for current state and the 3D data */
static int model;
static int roll=1;

/* 
 * Basic OpenGL initialization
 * It turns on lighting, back face culling, depth
 * buffer, etc. 
 *
 */
void init() {
  GLfloat global_ambient[4]={0.2, 0.2, 0.2, 1.0};
  GLfloat light0pos[4]=     {0.0, 0.0, 0.0, 1.0};
  GLfloat light0ambient[4]= {0.2, 0.2, 0.2, 1.0};
  GLfloat light0diffuse[4]= {0.6, 0.6, 0.6, 1.0};
  GLfloat light0specular[4]={0.8, 0.8, 0.8, 1.0};

  glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
  glutInitWindowSize(640,480);
  if (glutCreateWindow("_ glDino_space _") == GL_FALSE)
    exit(-1);

  glViewport(0, 0, 640, 480);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60, 4/3, 5,  2000);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glFrontFace(GL_CCW);

  glDisable(GL_DITHER);
  glShadeModel(GL_SMOOTH);

  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
  glLightfv(GL_LIGHT0, GL_POSITION, light0pos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0specular);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glMateriali(GL_FRONT, GL_SHININESS, 128);
  glClearColor(0.0, 0.0, 0.0, 1.0); 
}

static void key (unsigned char k, int x, int y) {
  switch(k) {
  case 27:
    exit(0);
    break;
  case 'q':
    moveForward();
    break;
  case 'w':
    moveBack();
    break;
  case 'y':
    roll = 1 - roll;
    break;
  case 'r':
    CameraReset();
    break;
  }
}

static void special (int k, int x, int y) {
  switch (k) {
  case GLUT_KEY_UP:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT) {
      moveUp();
    }
    else {
      pitchDown();
    }
    break;
  case GLUT_KEY_DOWN:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT) {
      moveDown();
    }
    else {
      pitchUp();
    }
    break;
  case GLUT_KEY_LEFT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT) {
      moveLeft();
    }
    else {
      if(roll)
	rollLeft();
      else
	yawRight();
    }
    break;
  case GLUT_KEY_RIGHT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT) {
      moveRight();
    }
    else {
      if(roll)
	rollRight();
      else
	yawLeft();
    }
    break;
  }
}

/* 
 * Here we actually call the display list we initialized in main().
 * The global variable model is the display list ID number that we
 * got when we initialized it.
 *
 */
static void draw() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPushMatrix();

  glCallList(model);

  glPopMatrix();
  glFlush();
  glutSwapBuffers();
}

int main(int argc, char **argv) {
  init();

  /* 
   * Here's the function call to load and compile the display list
   * data. It returns an int as the display list ID number.
   *
   */
  model = GL3DS_initialize_hubble();
  if (model == -1) {
    printf("Error initializing display list\n");
    exit (-1);
  }

  CameraReset();

  glutKeyboardFunc(key);
  glutSpecialFunc(special);
  glutDisplayFunc(draw);
  glutIdleFunc(draw);
  glutMainLoop();

  return(0);
}






